--lua scrabble by fullerlee@hotmail.com
--options handling by fb@frank-buss.de

--release history
--
--11th Nov 2005	0.1	Bug Fix & Feature
--3rd Nov 2005	0.1	Initial

--TODO
--* 1 hide the non-playing player's letters
--* 1 option for 1 player
--* 1 enforce letter placement
--2 dictionary with override/add
--* 2 timer option
--*  2 hold down R shoulder button for scoring i.e. show letter values & premium letter scores
--* 2 bonus for using all letters
--3 store word history
--* 3 hold down L shoulder button for help
--* 3 Document all functions
--2 Ability to mix up the letters
--2 Option to display what would be the score for the current placed tiles.
--2 Ability to choose a save game
--3 Pre-packaged save games for 1 player mode.
--STORE GAMESTATE IN SAME WAY AS POSSIBLE OPTIONS. ONLY REPAINT THE SCREEN IF GAMESTATE HAS CHANGED OR GAMEPAD HAS CHANGED

dofile "helperFuncs.lua"
dofile "timer.lua"
-------------------------------------
--options
possibleOptions = {
	numPlayers = {
		displayName = "Players",
		values = { 1, 2} },
	numLetters = {
		displayName = "Letters",
		values = { 7, 9 } },
	sound = {
		displayName = "Sound",
		values = { "on", "off" } },
	bonus = {
		displayName = "Bonus",
		values = { "on", "off" } },
	timer = {
		displayName = "Timer",
		values = { "on", "off" } }
}

-- default options
options = { numPlayers = 2, numLetters = 7, sound = "on", bonus = "on", timer = "off"}
selectedOptionNumber = 0
--------------------------------------------

--constants
SCREEN_WIDTH = 480
SCREEN_HEIGHT = 272
TILE_WIDTH = 18
TILE_HEIGHT = 18
BOARD_WIDTH = 15
BOARD_HEIGHT = 15
DOUBLE_LETTER = 1
DOUBLE_WORD = 2
TRIPLE_LETTER = 3
TRIPLE_WORD = 4
LETTER_COUNTS = {9,2,2,4,12,2,3,2,9,1,1,4,2,6,8,2,1,6,4,6,4,2,2,1,2,1,2}
--LETTER_COUNTS = {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1}
LETTER_VALUES = {1,3,3,2,1,4,2,4,1,8,5,1,3,1,1,3,10,1,1,1,1,4,4,8,4,10,0}
TOTAL_LETTERS = 100
P2_RACK_Y = 182
P1_RACK_Y = 112
P2_RACK_X = 296
P1_RACK_X = 296
SCORE_X = 330
SCORE_Y = 40
BOARD_CHAR_IDX = 0
BOARD_SQTYPE_IDX = 1
BOARD_STATUS_IDX = 2
BOARD_WORDDIR_IDX = 3
WORD_DIR_NONE = 0
WORD_DIR_HORIZ = 1
WORD_DIR_VERT = 3
WORD_DIR_BOTH = 4

--general globals
optionsFile = "scrabble.dat"
saveFile = "scrabble.sav"
board = {}
currentPlayer = 0
player1Letters = {}
player2Letters = {}
player1Score = 0
player2Score = 0
tileBag = {}
bagLettersRemaining = 0
currentcellx = 0
currentcelly = 0
currentRackSelection = 0

--colours
greenColour = Color.new(0, 255, 0)
whiteColour = Color.new(255,255,255)

--load the images
tileImage = Image.load("tiles_numbers.png")
gameBackgroundImage = Image.load("bg.png")
optionsImage = Image.load("options.png")
scoreInfoImage = Image.load("scoreInfo.png")
helpInfoImage = Image.load("helpInfo.png")
gameOverImage = Image.load("gameOver.png")
helpHelpImage = Image.load("helpHelp.png")
saveGameMenuImage = Image.load("saveGame.png")
tileFont = {}

-- load sounds
placeSound = Sound.load("place.wav")
moveSound = Sound.load("move.wav")
finishedSound = Sound.load("finished.wav")

------------------------------------------------------------------------------------------
--Initialize the board. Mark premium squares.
------------------------------------------------------------------------------------------
function initializeBoard()
	--board is a 3d array
	--1st	y
	--2nd	x
	--3rd	score, letter, tmp marker
	
	for y = 0,BOARD_HEIGHT-1 do
		board[y] = {}
		for x=0,BOARD_WIDTH-1 do
			board[y][x] = 0
			board[y][x] = {}
			board[y][x][BOARD_CHAR_IDX] = ""
			board[y][x][BOARD_SQTYPE_IDX] = 0
			board[y][x][BOARD_STATUS_IDX] = ""
			board[y][x][BOARD_WORDDIR_IDX] = WORD_DIR_NONE
		end
	end
	
	--add the premium squares
	--triple words
	board[0][0][BOARD_SQTYPE_IDX] = TRIPLE_WORD		--top left
	board[14][14][BOARD_SQTYPE_IDX] = TRIPLE_WORD	--bottom right
	board[0][14][BOARD_SQTYPE_IDX] = TRIPLE_WORD	--top right
	board[14][0][BOARD_SQTYPE_IDX] = TRIPLE_WORD	--bottom left
	board[0][7][BOARD_SQTYPE_IDX] = TRIPLE_WORD		--top middle
	board[7][0][BOARD_SQTYPE_IDX] = TRIPLE_WORD		--middle left
	board[7][14][BOARD_SQTYPE_IDX] = TRIPLE_WORD	--middle right
	board[14][7][BOARD_SQTYPE_IDX] = TRIPLE_WORD	--bottom middle
	
	--double words
	board[7][7][BOARD_SQTYPE_IDX] = DOUBLE_WORD	--centre square
	board[1][1][BOARD_SQTYPE_IDX] = DOUBLE_WORD	--
	board[2][2][BOARD_SQTYPE_IDX] = DOUBLE_WORD	--
	board[3][3][BOARD_SQTYPE_IDX] = DOUBLE_WORD	--
	board[4][4][BOARD_SQTYPE_IDX] = DOUBLE_WORD	--
	board[1][13][BOARD_SQTYPE_IDX] = DOUBLE_WORD
	board[2][12][BOARD_SQTYPE_IDX] = DOUBLE_WORD
	board[3][11][BOARD_SQTYPE_IDX] = DOUBLE_WORD
	board[4][10][BOARD_SQTYPE_IDX] = DOUBLE_WORD
	board[10][4][BOARD_SQTYPE_IDX] = DOUBLE_WORD
	board[11][3][BOARD_SQTYPE_IDX] = DOUBLE_WORD
	board[12][2][BOARD_SQTYPE_IDX] = DOUBLE_WORD
	board[13][1][BOARD_SQTYPE_IDX] = DOUBLE_WORD
	board[10][10][BOARD_SQTYPE_IDX] = DOUBLE_WORD
	board[11][11][BOARD_SQTYPE_IDX] = DOUBLE_WORD
	board[12][12][BOARD_SQTYPE_IDX] = DOUBLE_WORD
	board[13][13][BOARD_SQTYPE_IDX] = DOUBLE_WORD
	
	--triple letter
	board[1][5][BOARD_SQTYPE_IDX] = TRIPLE_LETTER
	board[1][9][BOARD_SQTYPE_IDX] = TRIPLE_LETTER
	board[5][1][BOARD_SQTYPE_IDX] = TRIPLE_LETTER
	board[5][5][BOARD_SQTYPE_IDX] = TRIPLE_LETTER
	board[5][9][BOARD_SQTYPE_IDX] = TRIPLE_LETTER
	board[5][13][BOARD_SQTYPE_IDX] = TRIPLE_LETTER
	board[9][1][BOARD_SQTYPE_IDX] = TRIPLE_LETTER
	board[9][5][BOARD_SQTYPE_IDX] = TRIPLE_LETTER
	board[9][9][BOARD_SQTYPE_IDX] = TRIPLE_LETTER
	board[9][13][BOARD_SQTYPE_IDX] = TRIPLE_LETTER
	board[13][5][BOARD_SQTYPE_IDX] = TRIPLE_LETTER
	board[13][9][BOARD_SQTYPE_IDX] = TRIPLE_LETTER
	
	--double letter
	board[0][3][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[0][11][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[2][6][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[2][8][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[3][0][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[3][14][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[3][7][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[6][2][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[6][6][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[6][8][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[6][12][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[7][3][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[7][11][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[8][2][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[8][6][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[8][8][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[8][12][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[11][0][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[11][14][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[11][7][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[12][6][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[12][8][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[14][3][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
	board[14][11][BOARD_SQTYPE_IDX] = DOUBLE_LETTER
end

------------------------------------------------------------------------------------------
--Initialize the letters.
------------------------------------------------------------------------------------------
function initializeLetters()
	bagLettersRemaining = 0
	for eachLetterCount in LETTER_COUNTS do
		for i=0,LETTER_COUNTS[eachLetterCount]-1 do
			tileBag[bagLettersRemaining] = string.char(string.byte("A") + (eachLetterCount-1))		
			bagLettersRemaining = bagLettersRemaining + 1
		end
	end
end

------------------------------------------------------------------------------------------
--Choose letters from the bag to fill <playerLetters> rack
------------------------------------------------------------------------------------------
function choosePlayerLetters(playerLetters)
	for i=0,options.numLetters-1 do
		if bagLettersRemaining==0 then			
			i = options.numLetters
		else
	
			if playerLetters[i] == nil or playerLetters[i][1] then
				playerLetters[i] = {}
				finished = false
				rnd = math.random(bagLettersRemaining)
				letNum = 0
				rndCount = 0
				while letNum < TOTAL_LETTERS and not finished do
					
					if tileBag[letNum]==nil then
					else
						rndCount = rndCount + 1
						if rndCount ==rnd then
							finished = true
							playerLetters[i][0] = tileBag[letNum]
							playerLetters[i][1] = false
							tileBag[letNum] = nil
							finished = true
							bagLettersRemaining = bagLettersRemaining -1
						end
					end
					letNum = letNum + 1
				end
			end
		end
	end
	
	if bagLettersRemaining==0 then
		--finished picking letters because bag is empty. close up the letters
		letCount = 0
		for x=0,options.numLetters-1 do
			if playerLetters[x] ~= nil and playerLetters[x][1]==false then
				playerLetters[letCount] = playerLetters[x]
				letCount = letCount + 1
			end
		end
		--clear up the end
		for x=letCount,options.numLetters-1 do
			playerLetters[x] = nil
		end
	end
	
end

------------------------------------------------------------------------------------------
--Change the selected square on the board according to xDir, yDir
------------------------------------------------------------------------------------------
function moveSelectedSquare(xDir, yDir)
	legalMove = false
	movesTried = 0
	oldx = currentcellx
	oldy = currentcelly
	while not legalMove and movesTried < BOARD_HEIGHT do
		currentcellx = currentcellx + xDir
		currentcelly = currentcelly + yDir
		
		if currentcellx < 0 then
			currentcellx = BOARD_WIDTH-1
		elseif currentcellx > BOARD_WIDTH-1 then
			currentcellx = 0
		end
		
		if currentcelly < 0 then
			currentcelly = BOARD_HEIGHT - 1
		elseif currentcelly > BOARD_HEIGHT-1 then
			currentcelly = 0
		end
		
		if board[currentcelly][currentcellx][0]=="" then
			legalMove = true
			
			board[oldy][oldx][BOARD_CHAR_IDX] = ""
			board[oldy][oldx][BOARD_STATUS_IDX] = ""
			
			board[currentcelly][currentcellx][BOARD_CHAR_IDX] = getCurrentRack()[currentRackSelection][0]
			--mark board as being tmp in case user wants to cancel the word
			board[currentcelly][currentcellx][BOARD_STATUS_IDX] = "T"
		else
			legalMove = false
		end
		movesTried = movesTried + 1
	end
end

------------------------------------------------------------------------------------------
--Initialize everything for a new game.
------------------------------------------------------------------------------------------
function initializeGame()
	player1Score = 0
	player2Score = 0

	currentcellx = 7
	currentcelly = 7
	
	lettersPlaced = 0
	
	initializeBoard()
	initializeLetters()
	
	choosePlayerLetters(player1Letters)
	choosePlayerLetters(player2Letters)
	
	currentPlayer = 1
end

------------------------------------------------------------------------------------------
--Paint the letters onto the board.
------------------------------------------------------------------------------------------
function drawBoard()
	--draw the board
	for y = 0,BOARD_HEIGHT-1 do
		boardLine = ""
		for x=0,BOARD_WIDTH-1 do
			if board[y][x][BOARD_CHAR_IDX] =="" then
				
			else
				drawChar(x*TILE_WIDTH, y*TILE_HEIGHT, board[y][x][BOARD_CHAR_IDX], tileFont)
				if y==currentcelly and x==currentcellx and gameState.onRack == false then
					drawSquareAroundTile(x*TILE_WIDTH, y*TILE_HEIGHT, greenColour)
				end
			end
		end
	end
	
end

------------------------------------------------------------------------------------------
--Return the rack of the current player
------------------------------------------------------------------------------------------
function getCurrentRack()
	if currentPlayer == 1 then
		currentRack = player1Letters
	else
		currentRack = player2Letters
	end
	return currentRack
end

------------------------------------------------------------------------------------------
--Can't remember what this does
------------------------------------------------------------------------------------------
function highlightcell(cellx, celly, isSelected)
	if isSelected then
		
		currentRack = getCurrentRack()
		
		letter = "A"
		for i=0,options.numLetters do
			if currentRack[i]==nil then
			
			else
				letter = currentRack[i][0]
			end
		end
		drawChar(cellx*TILE_WIDTH, celly*TILE_HEIGHT, letter, tileFont)
	else
	
		board[currentcelly][currentcellx][BOARD_CHAR_IDX] = getCurrentRack()[currentRackSelection][0]
		--screen:print(cellx * TILE_WIDTH + 8, celly * TILE_HEIGHT + 8, "O", greenColour)
	end
	
end

------------------------------------------------------------------------------------------
--Change the current selected tile on the rack according to dir
------------------------------------------------------------------------------------------
function changeRackSelection(dir)
	if dir == 0 then
		currentRackSelection = -1
		dir = 1
	end
	currentRack = getCurrentRack()
	moved = false
	over = 0
	while not moved do
		currentRackSelection = currentRackSelection+dir
		if currentRack[currentRackSelection] == nil or currentRack[currentRackSelection][1] == true then
			--nope
		else
			moved = true
		end
		
		if currentRackSelection > options.numLetters-1 then
			currentRackSelection = -1
			over=over+1
			if over>1 then
				moved = true
			end
		elseif currentRackSelection < 0 then
			currentRackSelection = options.numLetters
			over=over+1
			if over>1 then
				moved = true
			end
		end
		
	end

end

------------------------------------------------------------------------------------------
--Find the first available square on the board, starting at startx, starty.
------------------------------------------------------------------------------------------
function findFirstAvailableCell(startx, starty)
	found = false
	for y = starty,BOARD_HEIGHT-1 do
		for x=startx,BOARD_WIDTH-1 do
			if board[y][x][BOARD_CHAR_IDX]=="" then
				currentcellx = x
				currentcelly = y
				found = true
				x = BOARD_HEIGHT
				y = BOARD_WIDTH
			end
		end
	end
	--if haven't found, try again but from start of board
	if found==false then
		findFirstAvailableCell(0,0)
	end
end

------------------------------------------------------------------------------------------
--Paint the tiles on the racks.
------------------------------------------------------------------------------------------
function drawRacks()
	if currentPlayer == 1 then

		for eachLetter in player1Letters do
			if player1Letters[eachLetter][0] == "" or
				player1Letters[eachLetter][1] == true then
			else			
				drawChar(P1_RACK_X + (eachLetter * TILE_WIDTH), P1_RACK_Y, player1Letters[eachLetter][0], tileFont)
				if currentRackSelection==eachLetter then
					drawSquareAroundTile(P1_RACK_X + (eachLetter * TILE_WIDTH), P1_RACK_Y, greenColour)
				end
			end
		end
		
	elseif currentPlayer == 2 then
	
		for eachLetter in player2Letters do
			if player2Letters[eachLetter][0] == "" or
				player2Letters[eachLetter][1] == true then
			else
				drawChar(P2_RACK_X + (eachLetter * TILE_WIDTH), P2_RACK_Y, player2Letters[eachLetter][0], tileFont)
				if currentRackSelection==eachLetter then
					drawSquareAroundTile(P2_RACK_X + (eachLetter * TILE_WIDTH), P2_RACK_Y, greenColour)
				end
			end
		end
	end
end

------------------------------------------------------------------------------------------
--Change the current player.
------------------------------------------------------------------------------------------
function switchPlayer()
	if currentPlayer == 1 and options.numPlayers==2 then
		currentPlayer = 2
	else
		currentPlayer = 1
	end
end

------------------------------------------------------------------------------------------
--Calculate the score from the current turn.
------------------------------------------------------------------------------------------
function totalTurnScore()
	turnScore = 0
	wordTotal = 0
	for y = 0,BOARD_HEIGHT-1 do
		for x=0,BOARD_WIDTH-1 do
			if board[y][x][BOARD_STATUS_IDX] == "T" then
				--find other tiled squares touching this one & mark as part of the score
				--up
				newy = y
				finished = false
				while newy >= 0 and not finished do
	
					if newy==0 or board[newy-1][x][BOARD_CHAR_IDX]=="" then
						--if this is the 2nd letter, then word is allowable so mark current square
						if newy < y then
							if board[newy][x][BOARD_STATUS_IDX]=="T" then
							else
								board[newy][x][BOARD_STATUS_IDX] = "S"
							end
							if board[newy][x][BOARD_WORDDIR_IDX] == WORD_DIR_HORIZ then
								board[newy][x][BOARD_WORDDIR_IDX] = WORD_DIR_BOTH
							elseif board[newy][x][BOARD_WORDDIR_IDX] == WORD_DIR_NONE then
								board[newy][x][BOARD_WORDDIR_IDX] = WORD_DIR_VERT
							end
						end
						finished = true
					else
						if board[newy][x][BOARD_STATUS_IDX]=="T" then
						else
							board[newy][x][BOARD_STATUS_IDX] = "S"
						end
						if board[newy][x][BOARD_WORDDIR_IDX] == WORD_DIR_HORIZ then
							board[newy][x][BOARD_WORDDIR_IDX] = WORD_DIR_BOTH
						elseif board[newy][x][BOARD_WORDDIR_IDX] == WORD_DIR_NONE then
							board[newy][x][BOARD_WORDDIR_IDX] = WORD_DIR_VERT
						end
					end
					newy = newy - 1
				end
				
				--down
				newy = y
				finished = false
				while newy < BOARD_HEIGHT and not finished do
	
					if newy==BOARD_HEIGHT-1 or board[newy+1][x][BOARD_CHAR_IDX]=="" then
						--if this is the 2nd letter, then word is allowable so mark current square
						if newy > y then
							if board[newy][x][BOARD_STATUS_IDX]=="T" then
							else
								board[newy][x][BOARD_STATUS_IDX] = "S"
							end
							if board[newy][x][BOARD_WORDDIR_IDX] == WORD_DIR_HORIZ then
								board[newy][x][BOARD_WORDDIR_IDX] = WORD_DIR_BOTH
							elseif board[newy][x][BOARD_WORDDIR_IDX] == WORD_DIR_NONE then
								board[newy][x][BOARD_WORDDIR_IDX] = WORD_DIR_VERT
							end
						end
						finished = true
					else
						if board[newy][x][BOARD_STATUS_IDX]=="T" then
						else
							board[newy][x][BOARD_STATUS_IDX] = "S"
						end
						if board[newy][x][BOARD_WORDDIR_IDX] == WORD_DIR_HORIZ then
							board[newy][x][BOARD_WORDDIR_IDX] = WORD_DIR_BOTH
						elseif board[newy][x][BOARD_WORDDIR_IDX] == WORD_DIR_NONE then
							board[newy][x][BOARD_WORDDIR_IDX] = WORD_DIR_VERT
						end
					end
					newy = newy + 1
				end
				
				--left
				newx = x
				finished = false
				while newx >= 0 and not finished do
	
					if newx==0 or board[y][newx-1][BOARD_CHAR_IDX]=="" then
						--if this is the 2nd letter, then word is allowable so mark current square
						if newx < x then
							if board[y][newx][BOARD_STATUS_IDX]=="T" then
							else
								board[y][newx][BOARD_STATUS_IDX] = "S"
							end
							if board[y][newx][BOARD_WORDDIR_IDX] == WORD_DIR_VERT then
								board[y][newx][BOARD_WORDDIR_IDX] = WORD_DIR_BOTH
							elseif board[y][newx][BOARD_WORDDIR_IDX] == WORD_DIR_NONE then
								board[y][newx][BOARD_WORDDIR_IDX] = WORD_DIR_HORIZ
							end
						end
						finished = true
					else
						if board[y][newx][BOARD_STATUS_IDX]=="T" then
						else
							board[y][newx][BOARD_STATUS_IDX] = "S"
						end
						if board[y][newx][BOARD_WORDDIR_IDX] == WORD_DIR_VERT then
							board[y][newx][BOARD_WORDDIR_IDX] = WORD_DIR_BOTH
						elseif board[y][newx][BOARD_WORDDIR_IDX] == WORD_DIR_NONE then
							board[y][newx][BOARD_WORDDIR_IDX] = WORD_DIR_HORIZ
						end
					end
					newx = newx - 1
				end
				
				--right
				newx = x
				finished = false
				while newx < BOARD_WIDTH and not finished do
	
					if newx==BOARD_WIDTH-1 or board[y][newx+1][BOARD_CHAR_IDX]=="" then
						--if this is the 2nd letter, then word is allowable so mark current square
						if newx > x then
							if board[y][newx][BOARD_STATUS_IDX]=="T" then
							else
								board[y][newx][BOARD_STATUS_IDX] = "S"
							end
							if board[y][newx][BOARD_WORDDIR_IDX] == WORD_DIR_VERT then
								board[y][newx][BOARD_WORDDIR_IDX] = WORD_DIR_BOTH
							elseif board[y][newx][BOARD_WORDDIR_IDX] == WORD_DIR_NONE then
								board[y][newx][BOARD_WORDDIR_IDX] = WORD_DIR_HORIZ
							end
						end
						finished = true
					else
						if board[y][newx][BOARD_STATUS_IDX]=="T" then
						else
							board[y][newx][BOARD_STATUS_IDX] = "S"
						end
						if board[y][newx][BOARD_WORDDIR_IDX] == WORD_DIR_VERT then
							board[y][newx][BOARD_WORDDIR_IDX] = WORD_DIR_BOTH
						elseif board[y][newx][BOARD_WORDDIR_IDX] == WORD_DIR_NONE then
							board[y][newx][BOARD_WORDDIR_IDX] = WORD_DIR_HORIZ
						end
					end
					newx = newx + 1
				end
			end
		end
	end
	
	--now the board is marked with S squares.
	finished = false
	while not finished do
		for y = 0,BOARD_HEIGHT-1 do
			for x=0,BOARD_WIDTH-1 do
				if board[y][x][BOARD_STATUS_IDX] == "T" or
					board[y][x][BOARD_STATUS_IDX] == "S" then
					
					if board[y][x][BOARD_WORDDIR_IDX]==WORD_DIR_BOTH or
						board[y][x][BOARD_WORDDIR_IDX]==WORD_DIR_VERT then
											
						--check downwards
						multiplier = 1
						wordTotal = 0
						currentWord = ""
						for newy=y,BOARD_HEIGHT-1 do
							if board[newy][x][BOARD_STATUS_IDX]=="T" then
							
								currentWord = currentWord..board[newy][x][BOARD_CHAR_IDX]
							
								--T square so count premiums
								multiplier = multiplier * getWordMultiplier(board[newy][x][BOARD_SQTYPE_IDX])
								wordTotal = wordTotal + (getScoreForLetter(board[newy][x][BOARD_CHAR_IDX]) * getLetterMultiplier(board[newy][x][BOARD_SQTYPE_IDX]))
								if board[newy][x][BOARD_WORDDIR_IDX] == WORD_DIR_BOTH then
									board[newy][x][BOARD_WORDDIR_IDX] = WORD_DIR_HORIZ
								elseif board[newy][x][BOARD_WORDDIR_IDX] == WORD_DIR_VERT then
									board[newy][x][BOARD_WORDDIR_IDX] = WORD_DIR_NONE
								end
							elseif board[newy][x][BOARD_STATUS_IDX]=="S" then
								currentWord = currentWord..board[newy][x][BOARD_CHAR_IDX]
							
								wordTotal = wordTotal + getScoreForLetter(board[newy][x][BOARD_CHAR_IDX])
								if board[newy][x][BOARD_WORDDIR_IDX] == WORD_DIR_BOTH then
									board[newy][x][BOARD_WORDDIR_IDX] = WORD_DIR_HORIZ
								elseif board[newy][x][BOARD_WORDDIR_IDX] == WORD_DIR_VERT then
									board[newy][x][BOARD_WORDDIR_IDX] = WORD_DIR_NONE
								end
							else
								newy = BOARD_HEIGHT
							end
						
						end
						writeToDebug("WordTotal:"..wordTotal..", Word Mult:"..multiplier)
						wordTotal = wordTotal * multiplier
						writeToDebug("Word:"..currentWord.." Score:"..wordTotal)
						writeToDebug("")
					end
					
					if board[y][x][BOARD_WORDDIR_IDX]==WORD_DIR_BOTH or
						board[y][x][BOARD_WORDDIR_IDX]==WORD_DIR_HORIZ then
						
						--check right
						multiplier = 1
						wordTotal = 0
						currentWord = ""
						for newx=x,BOARD_WIDTH-1 do
							if board[y][newx][BOARD_STATUS_IDX]=="T" then
								currentWord = currentWord..board[y][newx][BOARD_CHAR_IDX]
								
								--T square so count premiums
								multiplier = multiplier * getWordMultiplier(board[y][newx][BOARD_SQTYPE_IDX])
								wordTotal = wordTotal + (getScoreForLetter(board[y][newx][BOARD_CHAR_IDX]) * getLetterMultiplier(board[y][newx][BOARD_SQTYPE_IDX]))
								if board[y][newx][BOARD_WORDDIR_IDX] == WORD_DIR_BOTH then
									board[y][newx][BOARD_WORDDIR_IDX] = WORD_DIR_VERT
								elseif board[y][newx][BOARD_WORDDIR_IDX] == WORD_DIR_HORIZ then
									board[y][newx][BOARD_WORDDIR_IDX] = WORD_DIR_NONE
								end
							elseif board[y][newx][BOARD_STATUS_IDX]=="S" then
								currentWord = currentWord..board[y][newx][BOARD_CHAR_IDX]
							
								wordTotal = wordTotal + getScoreForLetter(board[y][newx][BOARD_CHAR_IDX])
								if board[y][newx][BOARD_WORDDIR_IDX] == WORD_DIR_BOTH then
									board[y][newx][BOARD_WORDDIR_IDX] = WORD_DIR_VERT
								elseif board[y][newx][BOARD_WORDDIR_IDX] == WORD_DIR_HORIZ then
									board[y][newx][BOARD_WORDDIR_IDX] = WORD_DIR_NONE
								end
							else
								newx = BOARD_WIDTH
							end
						
						end
						writeToDebug("WordTotal:"..wordTotal..", Word Mult:"..multiplier)
						wordTotal = wordTotal * multiplier
						writeToDebug("Word:"..currentWord.." Score:"..wordTotal)
						writeToDebug("")
					end
					turnScore = turnScore + wordTotal
					wordTotal = 0
				end
			end
		end
		finished = true
	end

	--unmark all T and S squres
	removeBoardMarkers(false)
	
	if lettersPlaced >= 7 and options.bonus == "on" then
		turnScore = turnScore + 50
		writeToDebug("BONUS!")
	end
	
	--writeToDebug("Turn score: "..turnScore)
	return turnScore
end

------------------------------------------------------------------------------------------
--Clear all the temp board markers.
------------------------------------------------------------------------------------------
function removeBoardMarkers(removeTempChars)
	--unmark all T and S squres
	for y = 0,BOARD_HEIGHT-1 do
		for x=0,BOARD_WIDTH-1 do
			if removeTempChars and board[y][x][BOARD_STATUS_IDX] =="T" then
				board[y][x][BOARD_CHAR_IDX] = ""
			end
			board[y][x][BOARD_STATUS_IDX] = ""
			board[y][x][BOARD_WORDDIR_IDX] = WORD_DIR_NONE
		end
	end
end

------------------------------------------------------------------------------------------
--
------------------------------------------------------------------------------------------
function getWordMultiplier(squareType)
	mult = 1
	if squareType == DOUBLE_WORD then
		mult = 2
	elseif squareType == TRIPLE_WORD then
		mult = 3
	end
	return mult
end

------------------------------------------------------------------------------------------
--
------------------------------------------------------------------------------------------
function getLetterMultiplier(squareType)
	mult = 1
	if squareType == DOUBLE_LETTER then
		mult = 2
	elseif squareType == TRIPLE_LETTER then
		mult = 3
	end
	return mult
end

------------------------------------------------------------------------------------------
--Return the score for <letter>
------------------------------------------------------------------------------------------
function getScoreForLetter(letter)
	return LETTER_VALUES[(string.byte(letter) - string.byte("A"))+1]
end

------------------------------------------------------------------------------------------
--Perform end-of-turn logic.
------------------------------------------------------------------------------------------
function endTurn()
	if currentPlayer == 1 then
		player1Score = player1Score + totalTurnScore()
	else
		player2Score = player2Score + totalTurnScore()
	end
	choosePlayerLetters(getCurrentRack())
	lettersPlaced = 0
	switchPlayer()
	gameState.firstGo = false
	gameState.wordError = ""
	if options.timer=="on" then startTimer() end
end

------------------------------------------------------------------------------------------
--Draw an unfilled square around the tile <tileX> <tileY> in colour <colour>
------------------------------------------------------------------------------------------
function drawSquareAroundTile(tileX, tileY, colour)
	--top
	screen:drawLine(tileX, tileY, tileX + TILE_WIDTH, tileY, greenColour)
	--left
	screen:drawLine(tileX, tileY, tileX, tileY+TILE_HEIGHT, greenColour)
	--right
	screen:drawLine(tileX + TILE_WIDTH-1, tileY, tileX + TILE_WIDTH-1, tileY+TILE_HEIGHT, greenColour)
	--bottom
	screen:drawLine(tileX, tileY+TILE_HEIGHT, tileX + TILE_WIDTH, tileY+TILE_HEIGHT, greenColour)
end

------------------------------------------------------------------------------------------
--Paint the scores.
------------------------------------------------------------------------------------------
function drawScores()
	screen:print(SCORE_X, SCORE_Y, player1Score, greenColour)
	
	if options.numPlayers == 2 then
		screen:print(SCORE_X + 30, SCORE_Y, player2Score, greenColour)
	end
	
	if currentPlayer == 1 then
		screen:print(SCORE_X, SCORE_Y-10, "***", greenColour)
	else
		screen:print(SCORE_X + 30, SCORE_Y-10, "***", greenColour)
	end
	
	--draw letters remaining
	screen:print(SCORE_X-10, SCORE_Y+30, "Tiles left: "..bagLettersRemaining, greenColour)
end

------------------------------------------------------------------------------------------
--Return true if a game over condition has been met.
------------------------------------------------------------------------------------------
function checkForGameOver()
	gameOver = false
	if bagLettersRemaining == 0 then
		letterCount = 0
		for i=0,options.numLetters do
			if player1Letters[i] == nil or player1Letters[i][1] then
				
			else
				letterCount = letterCount + 1
			end
		end
		if letterCount == 0 then
			gameOver = true
		else
			letterCount = 0
			for i=0,options.numLetters do
				if player2Letters[i] == nil or player2Letters[i][1] then
				
				else
					letterCount = letterCount + 1
				end
			end
			if letterCount == 0 then
				gameOver = true
			end
			
		end
	end

	return gameOver
end

------------------------------------------------------------------------------------------
--Paint the title screen
------------------------------------------------------------------------------------------
function drawTitleScreen()
	screen:blit(0,0,gameBackgroundImage)
	x = TILE_WIDTH*3
	y = TILE_HEIGHT*7
	drawChar(x, y, "[", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, y, "S", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, y, "C", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, y, "R", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, y, "A", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, y, "B", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, y, "B", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, y, "L", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, y, "E", tileFont)
	
	x = P1_RACK_X
	drawChar(x, P1_RACK_Y, "X", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, P1_RACK_Y, "[", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, P1_RACK_Y, "S", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, P1_RACK_Y, "T", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, P1_RACK_Y, "A", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, P1_RACK_Y, "R", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, P1_RACK_Y, "T", tileFont)
	
	x = P2_RACK_X 
	drawChar(x, P2_RACK_Y, "O", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, P2_RACK_Y, "[", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, P2_RACK_Y, "O", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, P2_RACK_Y, "P", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, P2_RACK_Y, "T", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, P2_RACK_Y, "I", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, P2_RACK_Y, "O", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, P2_RACK_Y, "N", tileFont)
	x = x + TILE_WIDTH
	drawChar(x, P2_RACK_Y, "S", tileFont)
end

------------------------------------------------------------------------------------------
--Perform the logic for a game over state.
------------------------------------------------------------------------------------------
function doGameOver()
	--adjust scores for remaining letters
	if options.numPlayers == 2 then
	
		if currentPlayer == 1 then
			--p1 finished first, so add on p2s letters
			scoreDiff = 0
			for i=0,options.numLetters do
				if player2Letters[i] == nil or player2Letters[i][2] then
				
				else
					scoreDiff = scoreDiff + getScoreForLetter(player2Letters[i][0])
				end
			end
			player1Score = player1Score + scoreDiff
			player2Score = player2Score - scoreDiff
			
		elseif currentPlayer == 2 then
			--p2 finished first, so add on p2s letters
			scoreDiff = 0
			for i=0,options.numLetters do
				if player1Letters[i] == nil or player1Letters[i][2] then
				
				else
					scoreDiff = scoreDiff + getScoreForLetter(player1Letters[i][0])
				end
			end
			player2Score = player2Score + scoreDiff
			player1Score = player1Score - scoreDiff
		end
		
	end
end
 
------------------------------------------------------------------------------------------
--Paint the options screen.
--This method copied from 'snake' by fb@frank-buss.de
------------------------------------------------------------------------------------------
function drawOptionsScreen()
	deactiveColor = Color.new(0, 255, 0)
	activeColor = Color.new(255, 255, 255)
	x0 = 80
	y0 = 80
	black = Color.new(0, 0, 0)
	optionsx = (SCREEN_WIDTH/2)-(optionsImage:width()/2)
	optionsy = (SCREEN_HEIGHT/2)-(optionsImage:height()/2)
	screen:blit(optionsx,optionsy,optionsImage)
	y = 0
	selectedOptionValues = nil
	selectedOptionIndex = nil
	selectedOptionName = nil
	for optionKey, possibleOption in possibleOptions do
		color = deactiveColor
		if selectedOptionNumber == y then
			color = activeColor
			selectedOptionValues = possibleOption.values
		end
		screen:print(optionsx + 16, y * 10 + optionsy + 60, possibleOption.displayName, color)
		for i, value in possibleOption.values do
			color = deactiveColor
			if options[optionKey] == value then
				color = activeColor
				if selectedOptionNumber == y then
					selectedOptionIndex = i
					selectedOptionName = optionKey
				end
			end
			screen:print(optionsx + 60 + (i*70), y * 10 + optionsy + 60, value, color)
		end
		y = y + 1
	end

	quitOptions = false
	pad = Controls.read()
	if pad:circle() then 
		--don't save
		quitOptions = true
		Controls.waitpadup()
	end
	if pad:cross() then
		saveOptions()
		quitOptions = true
		Controls.waitpadup()
	end
	if pad:up() then
		selectedOptionNumber = selectedOptionNumber - 1
		if selectedOptionNumber < 0 then 
			selectedOptionNumber = y - 1 
		end
		Controls.waitpadup()
	end
	if pad:down() then
		selectedOptionNumber = selectedOptionNumber + 1
		if selectedOptionNumber >= y then 
			selectedOptionNumber = 0 
		end
		Controls.waitpadup()
	end
	len = 2
	if pad:left() then
		selectedOptionIndex = selectedOptionIndex - 1
		if selectedOptionIndex <= 0 then selectedOptionIndex = len end
		options[selectedOptionName] = selectedOptionValues[selectedOptionIndex]
		Controls.waitpadup()
	end
	if pad:right() then
		selectedOptionIndex = selectedOptionIndex + 1
		if selectedOptionIndex > len then selectedOptionIndex = 1 end
		options[selectedOptionName] = selectedOptionValues[selectedOptionIndex]
		Controls.waitpadup()
	end
	return quitOptions
end

------------------------------------------------------------------------------------------
--Play <soundToPlay> if the sound option is turned on.
------------------------------------------------------------------------------------------
function playSound(soundToPlay)

	if options.sound == "on" then
		soundToPlay:play()
	end

end

------------------------------------------------------------------------------------------
--Blit <imageToBlit> in the centre of the screen.
------------------------------------------------------------------------------------------
function blitInCentre(imageToBlit)
	screen:blit((SCREEN_WIDTH/2)-(imageToBlit:width()/2),(SCREEN_HEIGHT/2)-(imageToBlit:height()/2),imageToBlit)
end

------------------------------------------------------------------------------------------
--Cancel the current word, return tiles to the rack.
------------------------------------------------------------------------------------------
function cancelWord()
	--remove all markers from the board
	removeBoardMarkers(true)
	--mark all player letters as unused
	for i=0,options.numLetters do
		if getCurrentRack()[i] ~= nil then
			if getCurrentRack()[i][0] ~= "" then
				getCurrentRack()[i][1] = false
			end
		end
	end
	lettersPlaced = 0
end

------------------------------------------------------------------------------------------
--Write the status of the current word to the screen.
------------------------------------------------------------------------------------------
function writeWordStatus(statusMsg)
	screen:print(280, 260, statusMsg, greenColour)
end

------------------------------------------------------------------------------------------
--Return true if the current played word is legal.
------------------------------------------------------------------------------------------
function checkLegalMove()
	isLegalMove = false
	stuckToS = false
	dir = 0
	
	lastx = -1
	lasty = -1
	
	if gameState.firstGo then
		isLegalMove = true
	else
		for y = 0,BOARD_HEIGHT-1 do
			for x=0,BOARD_WIDTH-1 do
			
				if board[y][x][BOARD_STATUS_IDX] == "T" then
					if stuckToS == false then
						if y==0 then legalup=false else legalup = (board[y-1][x][BOARD_CHAR_IDX]~="" and board[y-1][x][BOARD_STATUS_IDX]~="T") end
						if y==BOARD_HEIGHT-1 then legaldown=false else legaldown = (board[y+1][x][BOARD_CHAR_IDX]~=""and board[y+1][x][BOARD_STATUS_IDX]~="T") end
						if x==0 then legalleft=false else legalleft = (board[y][x-1][BOARD_CHAR_IDX]~="" and board[y][x-1][BOARD_STATUS_IDX]~="T") end
						if x==BOARD_WIDTH-1 then legalright=false else legalright = (board[y][x+1][BOARD_CHAR_IDX]~="" and board[y][x+1][BOARD_STATUS_IDX]~="T") end
					
						if (legalup or legaldown or legalleft or legalright) then
							stuckToS = true
						end
					end
				
					if lastx==-1 and lasty==-1 then
						isLegalMove = true
						lastx = x
						lasty = y
					elseif lasty==y and (dir==1 or dir==0) then
						dir=1
						isLegalMove = true
						lastx = x
						lasty = y
					elseif lastx==x and (dir==2 or dir==0) then
						dir=2
						isLegalMove = true
						lastx = x
						lasty = y
					else
						isLegalMove = false
						x = BOARD_HEIGHT
						y = BOARD_WIDTH
					end
					
				end
			
			end
		end
		isLegalMove = (isLegalMove and stuckToS)
	end
	
	return isLegalMove
end

------------------------------------------------------------------------------------------
--Return boolvalue as a string.
------------------------------------------------------------------------------------------
function getBoolString(boolvalue)
	if boolvalue then return "true" else return "false" end
end

------------------------------------------------------------------------------------------
--Save the current gameState
------------------------------------------------------------------------------------------
function saveGame()
	file = io.open(saveFile, "w")
	--board
	file:write("board=")
	for y = 0,BOARD_HEIGHT-1 do
		for x=0,BOARD_WIDTH-1 do
			if board[y][x][BOARD_CHAR_IDX]=="" then char=" " else char = board[y][x][BOARD_CHAR_IDX] end
			if board[y][x][BOARD_SQTYPE_IDX]=="" then sqtype=" " else sqtype = board[y][x][BOARD_SQTYPE_IDX] end
			if board[y][x][BOARD_STATUS_IDX]=="" then status=" " else status = board[y][x][BOARD_STATUS_IDX] end
			if board[y][x][BOARD_WORDDIR_IDX]=="" then worddir=" " else worddir = board[y][x][BOARD_WORDDIR_IDX] end
			file:write(char..sqtype..status..worddir)
		end
	end
	file:write("\n")
	
	--letterbag
	file:write("bag=")
	for i=0,TOTAL_LETTERS-1 do
		if tileBag[i]==nil then
			file:write(" ")
		else
			file:write(tileBag[i])
		end
	end
	file:write("\n")
	
	--options
	file:write("numPlayers=")
	file:write(options.numPlayers)
	file:write("\n")
	file:write("numLetters=")
	file:write(options.numLetters)
	file:write("\n")
	
	--scores
	file:write("player1Score=")
	file:write(player1Score)
	file:write("\n")
	file:write("player2Score=")
	file:write(player2Score)
	file:write("\n")
	
	--otherGameInfo
	file:write("bagLettersRemaining=")
	file:write(bagLettersRemaining)
	file:write("\n")
	file:write("currentcellx=")
	file:write(currentcellx)
	file:write("\n")
	file:write("currentcelly=")
	file:write(currentcelly)
	file:write("\n")
	file:write("currentRackSelection=")
	file:write(currentRackSelection)
	file:write("\n")

	--playerracks
	--p1
	file:write("p1Rack=")
	for i=0,options.numLetters-1 do
		if player1Letters[i] == nil then
			file:write("  ")
		else
			file:write(player1Letters[i][0])
			if player1Letters[i][1] then
				file:write("Y")
			else
				file:write("N")
			end
		end
	end
	file:write("\n")
	--p2
	file:write("p2Rack=")
	for i=0,options.numLetters-1 do
		if player2Letters[i] == nil then
			file:write("  ")
		else
			file:write(player2Letters[i][0])
			if player2Letters[i][1] then
				file:write("Y")
			else
				file:write("N")
			end
		end
	end
	file:write("\n")
	
	--gameFlags
	file:write("gameState.menuOn=")
	if gameState.menuOn then file:write("Y") else file:write("N") end
	file:write("\n")
	file:write("gameState.gameUnderway=")
	if gameState.gameUnderway then file:write("Y") else file:write("N") end
	file:write("\n")
	file:write("gameState.newGame=")
	if gameState.newGame then file:write("Y") else file:write("N") end
	file:write("\n")
	file:write("gameState.gameOver=")
	if gameState.gameOver then file:write("Y") else file:write("N") end
	file:write("\n")
	file:write("gameState.onRack=")
	if gameState.onRack then file:write("Y") else file:write("N") end
	file:write("\n")
	file:write("gameState.firstGo=")
	if gameState.firstGo then file:write("Y") else file:write("N") end
	file:write("\n")
	file:write("gameState.wordError=")
	file:write(gameState.wordError)
	file:write("\n")
	
	writeToDebug("Saved game")
end

------------------------------------------------------------------------------------------
--Return <stringValue> as a boolean.
------------------------------------------------------------------------------------------
function getBoolFromString(stringValue)
	if stringValue == "Y" then
		return true
	else
		return false
	end
end

------------------------------------------------------------------------------------------
--Populate the rack <rack> with the string representation <stringValue>
------------------------------------------------------------------------------------------
function populateRackFromString(stringValue, rack)
	charnum = 1
	for i=0,options.numLetters-1 do
		char = string.sub(stringValue, charnum, charnum)
		if char==" " then
			rack[i] = nil
			charnum = charnum + 1
		else
			if rack[i] == nil then rack[i] = {} end
			rack[i][0] = char
			charnum = charnum + 1
			char = string.sub(stringValue, charnum, charnum)
			rack[i][1] = getBoolFromString(char)
		end
		charnum = charnum + 1
	end
end

------------------------------------------------------------------------------------------
--Load the saved game state.
------------------------------------------------------------------------------------------
function loadGame()
	file = io.open(saveFile, "r")
	if file then
		linenum = 0
		for line in file:lines() do
			equal = string.find(line, "=")
			if equal then
				lineName = string.sub(line, 1, equal - 1)
				lineValue = string.sub(line, equal + 1)
				
				if lineName=="board" then
					--board
					i = 1
					for y = 0,BOARD_HEIGHT-1 do
						if board[y] == nil then board[y] = {} end
						for x=0,BOARD_WIDTH-1 do
							if board[y][x] == nil then board[y][x] = {} end
							char = string.sub(lineValue, i, i)
							if char==" " then 
								board[y][x][0] = ""
							else 
								board[y][x][0] = char
							end
							i = i + 1
							char = string.sub(lineValue, i, i)
							if char==" " then board[y][x][1] = "" else board[y][x][i-1] = char end
							i = i + 1
							char = string.sub(lineValue, i, i)
							if char==" " then board[y][x][2] = "" else board[y][x][i-1] = char end
							i = i + 1
							char = string.sub(lineValue, i, i)
							if char==" " then board[y][x][3] = "" else board[y][x][i-1] = char end
							i = i + 1
						end
					end

				elseif lineName == "bag" then
				
				for i=1,TOTAL_LETTERS do
					char = string.sub(lineValue, i, i)
					if char == " " then
						tileBag[i-1] = nil
					else
						tileBag[i-1] = char
					end
				end
				
				elseif lineName == "numPlayers" then
					options.numPlayers = tonumber(lineValue)
				elseif lineName == "numLetters" then
					options.numLetters = tonumber(lineValue)
				elseif lineName == "player1Score" then
					player1Score = tonumber(lineValue)
				elseif lineName == "player2Score" then
					player2Score = tonumber(lineValue)
				elseif lineName == "bagLettersRemaining" then
					bagLettersRemaining = tonumber(lineValue)
				elseif lineName == "currentcellx" then
					currentcellx = tonumber(lineValue)
				elseif lineName == "currentcelly" then
					currentcelly = tonumber(lineValue)
				elseif lineName == "currentRackSelection" then
					currentRackSelection = tonumber(lineValue)
				elseif lineName == "p1Rack" then
					populateRackFromString(lineValue, player1Letters)
				elseif lineName == "p2Rack" then
					populateRackFromString(lineValue, player2Letters)
				elseif lineName == "gameState.menuOn" then
					gameState.menuOn = getBoolFromString(lineValue)
				elseif lineName == "gameState.gameUnderway" then
					gameState.gameUnderway = getBoolFromString(lineValue)
				elseif lineName == "gameState.newGame" then
					gameState.newGame = getBoolFromString(lineValue)
				elseif lineName == "gameState.gameOver" then
					gameState.gameOver = getBoolFromString(lineValue)
				elseif lineName == "gameState.onRack" then
					gameState.onRack = getBoolFromString(lineValue)
				elseif lineName == "gameState.firstGo" then
					gameState.firstGo = getBoolFromString(lineValue)
				elseif lineName == "gameState.wordError" then
					gameState.wordError = lineValue
				end
				
			end
		
			if linenum==0 then
				
			else --...
			end
			linenum = linenum + 1
		end
		return true
	else
		return false
	end
		
end

------------------------------------------------------------------------------------------
--Rearrange the letters on the current rack.
------------------------------------------------------------------------------------------
function mixUpCurrentRack()
	rack = getCurrentRack()
	newRack = {}

	letCount = 0
	for i=0,options.numLetters-1 do
		if rack[i] ~= nil then
			letCount = letCount + 1
		end	
	end

	for i=0,letCount-1 do
		letterPlaced = false
		while letterPlaced==false do
			rnd = math.random(letCount)-1
			if newRack[rnd] == nil then
				letterPlaced = true
				newRack[rnd] = rack[i]
			end
		end
	end
	
	setCurrentRack(newRack)
end

------------------------------------------------------------------------------------------
--Set <rack> as the current player's rack.
------------------------------------------------------------------------------------------
function setCurrentRack(rack)
	if currentPlayer==1 then
		player1Letters = rack
	else
		player2Letters = rack
	end
end

--main game
math.randomseed(os.time())
tileFont = loadTileFont()
loadOptions()

gameState = {
	menuOn = true,
	gameUnderway = false,
	newGame = false,
	gameOver = false,
	onRack = false,
	firstGo = false,
	saveMenuOn = false,
	wordError = ""
}

setTimeoutValue(180)

--main game loop
while not Controls.read():start() do

	if pad ~= Controls.read() then
		--writeToDebug("Change")
	end
	if (gameState.menuOn) then
		drawTitleScreen()
		pad = Controls.read()
		if pad:cross() then
			gameState.menuOn = false
			gameState.newGame = true
		elseif pad:circle() then
			gameState.menuOn = false
			optionsOn = true
		end
		Controls.waitpadup()
	elseif optionsOn then
		if drawOptionsScreen() then
			gameState.menuOn = true
			optionsOn = false
		end
	else
		screen:blit(0,0,gameBackgroundImage)
		screen:blit((SCREEN_WIDTH - helpHelpImage:width()) - 20, (SCREEN_HEIGHT - helpHelpImage:height())-20, helpHelpImage)
		
		if gameState.saveMenuOn then
			blitInCentre(saveGameMenuImage)
			pad = Controls.read()
			if pad:cross() then
				saveGame()
				gameState.saveMenuOn = false
				Controls.waitpadup()
			elseif pad:circle() then
				gameState.saveMenuOn = false
				Controls.waitpadup()
			elseif pad:triangle() then
				if loadGame() then
					gameState.saveMenuOn = false
				end
				Controls.waitpadup()
			end
			
		elseif gameState.newGame then		
			--setup for a new game
			initializeGame()
			gameState.newGame = false
			gameState.gameUnderway = true
			gameState.onRack  = true
			gameState.firstGo = true
			if options.timer=="on" then startTimer() end
		elseif gameState.gameUnderway then
			drawRacks()
			drawBoard()
			drawScores()
			
			if gameState.wordError ~= "" then
				writeWordStatus(gameState.wordError)
			end
			
			pad = Controls.read()
			
			if pad:r() then
				--show score info
				blitInCentre(scoreInfoImage)
				
			elseif pad:l() then
				--show help
				blitInCentre(helpInfoImage)
			
			elseif options.timer=="on" and timerIsFinished() then
				cancelWord()
				tf = gameState.firstGo
				endTurn()
				gameState.firstGo = tf
				changeRackSelection(0)
			
			elseif gameState.onRack then
				if pad:down() then
					--mix up the letters
					mixUpCurrentRack()
					Controls.waitpadup()
				elseif pad:up() then
					--go into change letters mode
				elseif pad:cross() then
					--select the letter and go into boardmode
					--mark the letter as being used
					playSound(placeSound)
					if currentRackSelection ~= -1 then
						getCurrentRack()[currentRackSelection][1] = true
						findFirstAvailableCell(currentcellx, currentcelly)
						moveSelectedSquare(0,0)
						gameState.onRack = false
					end
					Controls.waitpadup()
				elseif pad:circle() then
					cancelWord()
					gameState.onRack = true
					Controls.waitpadup()
				elseif pad:select() then
					gameState.saveMenuOn = true
					Controls.waitpadup()
				elseif pad:left() then
					playSound(moveSound)
					changeRackSelection(-1)
					Controls.waitpadup()
				elseif pad:right() then
					playSound(moveSound)
					changeRackSelection(1)
					Controls.waitpadup()
				elseif pad:triangle() then
					playSound(finishedSound)
					if checkLegalMove() == false then
						gameState.wordError = "Incorrect Play."
						cancelWord()
						gameState.onRack = true
					elseif board[7][7][BOARD_STATUS_IDX] ~= "T" and gameState.firstGo then
						--first go, but middle square is not used. reject word
						gameState.wordError = "Play centre square."
						cancelWord()
						gameState.onRack = true
					else
						--user has finished turn
						gameState.wordError = ""
						endTurn()
						changeRackSelection(0)
					end
					Controls.waitpadup()
				elseif pad:square() then
					--quit game
					--back to menu screen
					gameState.menuOn = true
					gameState.gameOver = false
					Controls.waitpadup()
				end
			else
				if pad:cross() then
					lettersPlaced = lettersPlaced + 1
					playSound(placeSound)
					changeRackSelection(1)
					gameState.onRack = true
					Controls.waitpadup()
				elseif pad:circle() then
					cancelWord()
					changeRackSelection(0)
					gameState.onRack = true
					Controls.waitpadup()
				elseif pad:select() then
					gameState.saveMenuOn = true
					Controls.waitpadup()
				elseif pad:up() then
					playSound(moveSound)
					moveSelectedSquare(0, -1)
					Controls.waitpadup()
				elseif pad:down() then
					playSound(moveSound)
					moveSelectedSquare(0, 1)
					Controls.waitpadup()
				elseif pad:left() then
					playSound(moveSound)
					moveSelectedSquare(-1, 0)
					Controls.waitpadup()
				elseif pad:right() then
					playSound(moveSound)
					moveSelectedSquare(1, 0)
					Controls.waitpadup()
				elseif pad:square() then
					--quit game
					--back to menu screen
					gameState.menuOn = true
					gameState.gameOver = false
					Controls.waitpadup()
				end
				if not gameState.onRack then
					highlightcell(currentcellx, currentcelly, false)
				else
					--check for a game over condition
					if checkForGameOver() then
						doGameOver()
						gameState.onRack = false
						gameState.gameOver = true
						gameState.gameUnderway = false
					end
				end
			end
		
		elseif gameState.gameOver then
			drawBoard()
			drawScores()
			screen:print(SCORE_X, SCORE_Y+15, "Game Over", greenColour)
			
			pad = Controls.read()
			if not pad:l() then
				infoTextx = (SCREEN_WIDTH/2)-(optionsImage:width()/2) + 60
				infoTexty = (SCREEN_HEIGHT/2)-(optionsImage:height()/2) + 60
				blitInCentre(gameOverImage)
				
				if options.numPlayers == 1 then
					screen:print(infoTextx, infoTexty, "Your final score was: "..player1Score, greenColour)
				else
					screen:print(infoTextx, infoTexty, "Player 1: "..player1Score, greenColour)
					screen:print(infoTextx, infoTexty+10, "Player 2: "..player2Score, greenColour)
					if player1Score > player2Score then
						screen:print(infoTextx, infoTexty+20, "Player 1 wins", greenColour)
					elseif player2Score > player1Score then
						screen:print(infoTextx, infoTexty+20, "Player 2 wins", greenColour)
					else
						screen:print(infoTextx, infoTexty+20, "It's a draw", greenColour)
					end
				end
			end
			if pad:cross() then
				--start again
				gameState.newGame = true
				Controls.waitpadup()			
			elseif pad:circle() then
				--back to menu screen
				gameState.menuOn = true
				gameState.gameOver = false
				Controls.waitpadup()
			end
		end
		if options.timer=="on" then screen:print(SCORE_X+60,SCORE_Y, getTimeLeft(), greenColour) end
	end
	screen.waitVblankStart()
	screen.flip()
end